import { __decorate, __spread, __param } from 'tslib';
import { ElementRef, Renderer2, Input, ViewChild, ContentChildren, forwardRef, HostBinding, HostListener, Component, Optional, Directive, EventEmitter, Output, NgModule } from '@angular/core';
import { StyleCollection, shadowBuilder, Positioning, LyOverlayPosition, XPosition, YPosition, LyTheme2, StyleRenderer, LyOverlay, LyCommonModule, LyOverlayModule } from '@alyle/ui';
import { trigger, transition, group, style, animate } from '@angular/animations';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { ViewportRuler } from '@angular/cdk/scrolling';
import { Subject, asapScheduler } from 'rxjs';
import { take, delay, debounceTime } from 'rxjs/operators';
import { FormsModule } from '@angular/forms';
import { CommonModule } from '@angular/common';

var STYLE_PRIORITY = -1;
var STYLES = function (theme, ref) {
    var menu = ref.selectorsOf(STYLES);
    var after = theme.after;
    return {
        $name: LyMenu.и,
        $priority: STYLE_PRIORITY,
        root: function () {
            var _a;
            return (((_a = theme.menu) === null || _a === void 0 ? void 0 : _a.root) && (theme.menu.root instanceof StyleCollection
                ? theme.menu.root.setTransformer(function (fn) { return fn(menu); }).css
                : theme.menu.root(menu)));
        },
        container: function (className) { return className + "{background:" + theme.background.primary.default + ";border-radius:2px;box-shadow:" + shadowBuilder(4) + ";display:block;padding-top:8px;padding-bottom:8px;transform-origin:inherit;pointer-events:all;overflow:auto;max-height:inherit;max-width:inherit;box-sizing:border-box;}"; },
        item: function (className) { return className + "{display:flex;min-height:48px;border-radius:0;width:100%;justify-content:flex-start;font-weight:400;}" + className + " ly-icon{margin-" + after + ":16px;}"; },
        itemSubMenuTrigger: function () { return function (className) { return className + "{padding-" + after + ":32px;}" + className + "::after{width:0;height:0;border-style:solid;border-width:5px 0 5px 5px;border-color:transparent transparent transparent currentColor;content:\"\";display:inline-block;position:absolute;top:50%;" + after + ":16px;transform:translateY(-50%);}"; }; }
    };
};
var ANIMATIONS = [
    trigger('transformMenu', [
        transition('void => enter', group([
            style({
                opacity: 0,
                transform: 'scale(0.8)'
            }),
            animate('100ms linear', style({
                opacity: 1
            })),
            animate('120ms cubic-bezier(0, 0, 0.2, 1)', style({ transform: 'scale(1)' })),
        ]))
    ]),
    trigger('transformMenuLeave', [
        transition('* => void', animate('100ms 25ms linear', style({ opacity: 0 })))
    ])
];
/** Menu container */
var LyMenu = /** @class */ (function () {
    function LyMenu(_theme, _el, _renderer, _viewportRuler, sRenderer) {
        this._theme = _theme;
        this._el = _el;
        this._renderer = _renderer;
        this._viewportRuler = _viewportRuler;
        this.sRenderer = sRenderer;
        /**
         * styles
         * @docs-private
         */
        this.classes = this.sRenderer.renderSheet(STYLES, true);
        /** Emits whenever an animation on the menu completes. */
        this._animationDone = new Subject();
        this._hasBackdrop = true;
    }
    LyMenu_1 = LyMenu;
    Object.defineProperty(LyMenu.prototype, "ref", {
        get: function () {
            return this._ref;
        },
        /** Menu Trigger */
        set: function (value) {
            this._ref = value;
            this._menuRef = value._menuRef;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LyMenu.prototype, "hasBackdrop", {
        /** Whether the menu has a backdrop. */
        get: function () {
            return this._hasBackdrop;
        },
        set: function (value) {
            this._hasBackdrop = coerceBooleanProperty(value);
        },
        enumerable: true,
        configurable: true
    });
    LyMenu.prototype.ngOnChanges = function () {
        var _a;
        if (((_a = this.ref) === null || _a === void 0 ? void 0 : _a._menuRef) && this._container) {
            // Update backdrop
            this.ref._menuRef.updateBackdrop(this.ref._isItemSubMenuTrigger() ? false : this.hasBackdrop);
            this._updatePlacement();
            this._checkBackdropAndOpenOnHover();
        }
    };
    LyMenu.prototype.ngOnInit = function () {
        if (!this.ref) {
            throw new Error('LyMenu: require @Input() ref');
        }
    };
    LyMenu.prototype.ngAfterViewInit = function () {
        var _this = this;
        if (this.ref._menuRef) {
            this.ref._menuRef.onResizeScroll = this._updatePlacement.bind(this);
            this.ref._menuRef.updateBackdrop(this.ref._isItemSubMenuTrigger() ? false : this.hasBackdrop);
            this._checkBackdropAndOpenOnHover();
        }
        this._updatePlacement();
        this.ref.menuOpened.emit();
        Promise.resolve(null).then(function () {
            _this.ref._setMenuOpenToTrue();
        });
        var hostTrigger = this._getHostMenuTrigger();
        hostTrigger._menuDetached
            .pipe(take(1))
            .subscribe(function () { return _this._ref.closeMenu(); });
        this._addOpenOnHover();
    };
    LyMenu.prototype.ngOnDestroy = function () {
        this._removeOpenOnHoverListeners();
    };
    LyMenu.prototype._checkBackdropAndOpenOnHover = function () {
        var _a;
        var hostTrigger = this._getHostMenuTrigger();
        if (this.hasBackdrop && ((_a = hostTrigger._menuOpenOnHoverRef) === null || _a === void 0 ? void 0 : _a.openOnHover)) {
            throw new Error(LyMenu_1.и + ": Can't use [hasBackdrop] with [openOnHover] at the same time, set [hasBackdrop] to false to use [openOnHover]");
        }
    };
    LyMenu.prototype._getHostMenuTrigger = function () {
        var _a;
        var menuTrigger = this.ref;
        while ((_a = menuTrigger._menu) === null || _a === void 0 ? void 0 : _a.ref) {
            menuTrigger = menuTrigger._menu.ref;
        }
        return menuTrigger;
    };
    LyMenu.prototype._addOpenOnHover = function () {
        var _a;
        var hostTrigger = this._getHostMenuTrigger();
        if (((_a = hostTrigger._menuOpenOnHoverRef) === null || _a === void 0 ? void 0 : _a.openOnHover) && !this._mouseenterListen && !this._mouseleaveListen) {
            hostTrigger._menuOpenOnHoverRef._handleMouseEnterOrLeave(true);
            this._mouseenterListen = this._renderer
                .listen(this._el.nativeElement, 'mouseenter', function () { return hostTrigger._menuOpenOnHoverRef._handleMouseEnterOrLeave(true); });
            this._mouseleaveListen = this._renderer
                .listen(this._el.nativeElement, 'mouseleave', function () { return hostTrigger._menuOpenOnHoverRef._handleMouseEnterOrLeave(false); });
        }
    };
    /** Remove listeners */
    LyMenu.prototype._removeOpenOnHoverListeners = function () {
        if (this._mouseenterListen) {
            this._mouseenterListen();
        }
        if (this._mouseleaveListen) {
            this._mouseleaveListen();
        }
    };
    /** Update Menu Position */
    LyMenu.prototype._updatePlacement = function () {
        var _a, _b;
        var el = (_a = this.ref._menuRef) === null || _a === void 0 ? void 0 : _a.containerElement;
        var container = (_b = this._container) === null || _b === void 0 ? void 0 : _b.nativeElement;
        // Do not update when not available
        if (!el || !container) {
            return;
        }
        // reset height & width
        this._renderer.setStyle(container, 'height', 'initial');
        this._renderer.setStyle(container, 'width', 'initial');
        var position = this.placement
            ? new Positioning(this.placement, this.xPosition, this.yPosition, this.ref._getHostElement(), el, this._theme.variables)
            : !this.ref._isItemSubMenuTrigger()
                ? new LyOverlayPosition(this._theme, this._viewportRuler, this.ref._getHostElement(), el)
                    .setXAnchor(this.xAnchor)
                    .setYAnchor(this.yAnchor)
                    .setXAxis(this.xAxis)
                    .setYAxis(this.yAxis)
                    .setFlip(true)
                    .build()
                : new LyOverlayPosition(this._theme, this._viewportRuler, this.ref._getHostElement(), el)
                    .setXAnchor(XPosition.after)
                    .setYAnchor(YPosition.above)
                    .setFlip(true)
                    .build();
        if (position instanceof Positioning) {
            // set position deprecated
            this._renderer.setStyle(el, 'transform', "translate3d(" + position.x + "px, " + position.y + "px, 0)");
            this._renderer.setStyle(this._el.nativeElement, 'transform-origin', position.ox + " " + position.oy + " 0");
            // set height & width deprecated
            this._renderer.setStyle(container, 'height', position.height === 'initial' ? '100%' : position.height);
            this._renderer.setStyle(container, 'width', position.width === 'initial' ? '100%' : position.width);
        }
        else {
            // set position
            this._renderer.setStyle(el, 'left', position.x + "px");
            this._renderer.setStyle(el, 'top', position.y + "px");
            this._renderer.setStyle(container, 'width', position.width ? position.width + "px" : '100%');
            this._renderer.setStyle(container, 'height', position.height ? position.height + "px" : '100%');
            this._renderer.setStyle(this._el.nativeElement, 'transform-origin', position.xo + "px " + position.yo + "px 0");
        }
    };
    LyMenu.prototype._onAnimationStart = function (event) {
        this._isAnimating = true;
        if (event.triggerName === 'transformMenuLeave' && event.toState === 'void') {
            this._isDestroying = true;
        }
    };
    LyMenu.prototype._onAnimationDone = function (event) {
        this._animationDone.next(event);
        this._isAnimating = false;
        if (event.toState === 'void' && event.triggerName === 'transformMenuLeave') {
            this.ref.destroy(this._menuRef);
        }
    };
    var LyMenu_1;
    LyMenu.ctorParameters = function () { return [
        { type: LyTheme2 },
        { type: ElementRef },
        { type: Renderer2 },
        { type: ViewportRuler },
        { type: StyleRenderer }
    ]; };
    /** @docs-private */
    LyMenu.и = 'LyMenu';
    __decorate([
        Input()
    ], LyMenu.prototype, "ref", null);
    __decorate([
        Input()
    ], LyMenu.prototype, "hasBackdrop", null);
    __decorate([
        ViewChild('container')
    ], LyMenu.prototype, "_container", void 0);
    __decorate([
        ContentChildren(forwardRef(function () { return LyMenuItem; }))
    ], LyMenu.prototype, "menuItems", void 0);
    __decorate([
        Input()
    ], LyMenu.prototype, "xAnchor", void 0);
    __decorate([
        Input()
    ], LyMenu.prototype, "yAnchor", void 0);
    __decorate([
        Input()
    ], LyMenu.prototype, "xAxis", void 0);
    __decorate([
        Input()
    ], LyMenu.prototype, "yAxis", void 0);
    __decorate([
        Input()
    ], LyMenu.prototype, "placement", void 0);
    __decorate([
        Input()
    ], LyMenu.prototype, "xPosition", void 0);
    __decorate([
        Input()
    ], LyMenu.prototype, "yPosition", void 0);
    __decorate([
        HostBinding('@transformMenuLeave')
    ], LyMenu.prototype, "transformMenuLeave", void 0);
    __decorate([
        HostListener('@transformMenuLeave.start', ['$event'])
    ], LyMenu.prototype, "_onAnimationStart", null);
    __decorate([
        HostListener('@transformMenuLeave.done', ['$event'])
    ], LyMenu.prototype, "_onAnimationDone", null);
    LyMenu = LyMenu_1 = __decorate([
        Component({
            selector: 'ly-menu',
            animations: __spread(ANIMATIONS),
            template: "<div #container\n  [class]=\"classes.container\"\n  [@transformMenu]=\"'enter'\"\n  (@transformMenu.start)=\"_onAnimationStart($event)\"\n  (@transformMenu.done)=\"_onAnimationDone($event)\"\n>\n  <ng-content></ng-content>\n</div>",
            exportAs: 'lyMenu',
            providers: [
                StyleRenderer
            ]
        })
    ], LyMenu);
    return LyMenu;
}());
var LyMenuItem = /** @class */ (function () {
    function LyMenuItem(_menu, el, renderer) {
        this._menu = _menu;
        renderer.addClass(el.nativeElement, _menu.classes.item);
    }
    LyMenuItem.prototype._handleClick = function () {
        var _a;
        if (this._menu.ref && this._menu.ref._menuRef) {
            if (!this._getItemSubMenuTrigger()) {
                var currentTrigger = this._menu.ref;
                while (currentTrigger) {
                    currentTrigger.closeMenu();
                    currentTrigger = (_a = currentTrigger._menu) === null || _a === void 0 ? void 0 : _a.ref;
                }
            }
        }
    };
    LyMenuItem.prototype._handleMouseEnter = function () {
        var _this = this;
        var itemSubMenuTrigger = this._getItemSubMenuTrigger();
        if (itemSubMenuTrigger && !this._menu._isDestroying) {
            if (this._menu._isAnimating) {
                this._menu._animationDone
                    .pipe(take(1), delay(0, asapScheduler))
                    .subscribe(function () {
                    itemSubMenuTrigger.openMenu();
                    _this._closeOtherMenus();
                });
            }
            else {
                itemSubMenuTrigger.openMenu();
                this._closeOtherMenus();
            }
        }
        else {
            this._closeOtherMenus();
        }
    };
    /** Except for this, close all menus */
    LyMenuItem.prototype._closeOtherMenus = function () {
        var _this = this;
        this._menu.menuItems.forEach(function (menuItem) {
            var _a;
            if (menuItem !== _this) {
                (_a = menuItem._getItemSubMenuTrigger()) === null || _a === void 0 ? void 0 : _a.closeMenu();
            }
        });
    };
    LyMenuItem.prototype._setItemSubMenuTrigger = function (menuTrigger) {
        this._itemSubMenuTrigger = menuTrigger;
    };
    LyMenuItem.prototype._getItemSubMenuTrigger = function () {
        return this._itemSubMenuTrigger;
    };
    LyMenuItem.ctorParameters = function () { return [
        { type: LyMenu, decorators: [{ type: Optional }] },
        { type: ElementRef },
        { type: Renderer2 }
    ]; };
    LyMenuItem = __decorate([
        Directive({
            selector: '[ly-menu-item]',
            host: {
                '(click)': '_handleClick()',
                '(mouseenter)': '_handleMouseEnter()'
            }
        }),
        __param(0, Optional())
    ], LyMenuItem);
    return LyMenuItem;
}());
var LyMenuTriggerFor = /** @class */ (function () {
    function LyMenuTriggerFor(elementRef, overlay, _menuItem, sRenderer, _menu) {
        this.elementRef = elementRef;
        this.overlay = overlay;
        this._menuItem = _menuItem;
        this.sRenderer = sRenderer;
        this._menu = _menu;
        this.classes = this.sRenderer.renderSheet(STYLES);
        this._menuOpen = false;
        this._menuDetached = new Subject();
        this.menuOpened = new EventEmitter();
        this.menuClosed = new EventEmitter();
        if (this._isItemSubMenuTrigger()) {
            _menuItem._setItemSubMenuTrigger(this);
            sRenderer.addClass(this.classes.itemSubMenuTrigger);
        }
    }
    Object.defineProperty(LyMenuTriggerFor.prototype, "menuOpen", {
        /** Whether the menu is open. */
        get: function () {
            return this._menuOpen;
        },
        enumerable: true,
        configurable: true
    });
    LyMenuTriggerFor.prototype.ngOnDestroy = function () {
        // Not force destruction if it is already being destroyed
        if (!this._destroying) {
            this.closeMenu();
        }
        this._menuDetached.complete();
    };
    LyMenuTriggerFor.prototype._handleClick = function () {
        if (!this._isItemSubMenuTrigger()) {
            this.toggleMenu();
        }
    };
    /** Opens the menu */
    LyMenuTriggerFor.prototype.openMenu = function () {
        if (!this._menuRef) {
            this._menuRef = this.overlay.create(this.lyMenuTriggerFor, {
                $implicit: this,
                data: this.menuData
            }, {
                styles: {
                    top: 0,
                    left: 0,
                    pointerEvents: null
                },
                fnDestroy: this.detach.bind(this),
                hasBackdrop: false
            });
        }
    };
    /** Closes the menu */
    LyMenuTriggerFor.prototype.closeMenu = function () {
        this.detach();
    };
    /** Toggle menu */
    LyMenuTriggerFor.prototype.toggleMenu = function () {
        if (this._menuRef) {
            this.closeMenu();
        }
        else {
            this.openMenu();
        }
    };
    /** @docs-private */
    LyMenuTriggerFor.prototype.detach = function () {
        if (this._menuRef) {
            this._menuRef.detach();
            this._menuRef = null;
            this._destroying = true;
            this._menuDetached.next();
        }
    };
    /** @docs-private */
    LyMenuTriggerFor.prototype.destroy = function (menuRef) {
        var _this = this;
        this.menuClosed.emit(null);
        menuRef.remove();
        this._destroying = false;
        Promise.resolve(null).then(function () { return _this._menuOpen = false; });
    };
    LyMenuTriggerFor.prototype._getHostElement = function () {
        return this.elementRef.nativeElement;
    };
    LyMenuTriggerFor.prototype._setMenuOpenToTrue = function () {
        this._menuOpen = true;
    };
    /**
     * @docs-private
     */
    LyMenuTriggerFor.prototype._isItemSubMenuTrigger = function () {
        return !!this._menuItem;
    };
    LyMenuTriggerFor.ctorParameters = function () { return [
        { type: ElementRef },
        { type: LyOverlay },
        { type: LyMenuItem, decorators: [{ type: Optional }] },
        { type: StyleRenderer },
        { type: LyMenu, decorators: [{ type: Optional }] }
    ]; };
    __decorate([
        Input()
    ], LyMenuTriggerFor.prototype, "lyMenuTriggerFor", void 0);
    __decorate([
        Input('lyMenuTriggerData')
    ], LyMenuTriggerFor.prototype, "menuData", void 0);
    __decorate([
        Output()
    ], LyMenuTriggerFor.prototype, "menuOpened", void 0);
    __decorate([
        Output()
    ], LyMenuTriggerFor.prototype, "menuClosed", void 0);
    LyMenuTriggerFor = __decorate([
        Directive({
            selector: '[lyMenuTriggerFor]',
            host: {
                '(click)': '_handleClick()'
            },
            exportAs: 'lyMenuTriggerFor',
            providers: [
                StyleRenderer
            ]
        }),
        __param(2, Optional()),
        __param(4, Optional())
    ], LyMenuTriggerFor);
    return LyMenuTriggerFor;
}());
var LyMenuOpenOnHover = /** @class */ (function () {
    function LyMenuOpenOnHover(_trigger) {
        var _this = this;
        this._trigger = _trigger;
        this._events = new Subject();
        this._openOnHover = true;
        _trigger._menuOpenOnHoverRef = this;
        this._events
            .pipe(debounceTime(200))
            .subscribe(function (enterOrLeave) {
            if (_this.openOnHover) {
                if (enterOrLeave) {
                    _trigger.openMenu();
                }
                else {
                    _trigger.closeMenu();
                }
            }
        });
    }
    Object.defineProperty(LyMenuOpenOnHover.prototype, "openOnHover", {
        /** Whether menu should open on hover. */
        get: function () {
            return this._openOnHover;
        },
        set: function (value) {
            var _this = this;
            this._openOnHover = coerceBooleanProperty(value);
            Promise.resolve(null)
                .then(function () { return _this._openOnHover
                ? _this._trigger._menuOpenOnHoverRef = _this
                : delete _this._trigger._menuOpenOnHoverRef; });
        },
        enumerable: true,
        configurable: true
    });
    LyMenuOpenOnHover.prototype.ngOnDestroy = function () {
        this._events.complete();
    };
    /** Handle mouseenter or mouseleave */
    LyMenuOpenOnHover.prototype._handleMouseEnterOrLeave = function (leaveOrEnter) {
        if (this.openOnHover) {
            if (leaveOrEnter) {
                this._trigger.openMenu();
            }
            this._events.next(leaveOrEnter);
        }
    };
    LyMenuOpenOnHover.ctorParameters = function () { return [
        { type: LyMenuTriggerFor }
    ]; };
    __decorate([
        Input()
    ], LyMenuOpenOnHover.prototype, "openOnHover", null);
    LyMenuOpenOnHover = __decorate([
        Directive({
            selector: '[lyMenuTriggerFor][openOnHover]',
            host: {
                '(mouseenter)': '_handleMouseEnterOrLeave(true)',
                '(mouseleave)': '_handleMouseEnterOrLeave(false)'
            }
        })
    ], LyMenuOpenOnHover);
    return LyMenuOpenOnHover;
}());

var LyMenuModule = /** @class */ (function () {
    function LyMenuModule() {
    }
    LyMenuModule = __decorate([
        NgModule({
            imports: [CommonModule, FormsModule, LyCommonModule, LyOverlayModule],
            exports: [LyMenu, LyMenuItem, LyMenuTriggerFor, LyMenuOpenOnHover],
            declarations: [LyMenu, LyMenuItem, LyMenuTriggerFor, LyMenuOpenOnHover],
        })
    ], LyMenuModule);
    return LyMenuModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { LyMenu, LyMenuItem, LyMenuModule, LyMenuOpenOnHover, LyMenuTriggerFor, STYLES };
//# sourceMappingURL=alyle-ui-menu.js.map
